import torch
from torchvision import datasets
from torchvision import transforms
import matplotlib.pyplot as plt
import time
device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
print(device)

tensor_transform = transforms.ToTensor()

dataset = datasets.MNIST(root = "./data",
                        train = True,
                        download = True, 
                        transform = tensor_transform)


loader = torch.utils.data.DataLoader(dataset = dataset,
                                    batch_size = 32,
                                    shuffle = True)

# Creating a PyTorch class
# 28*28 ==> 9 ==> 28*28
class AE(torch.nn.Module):
    def __init__(self):
        super().__init__()
         
        # Building an linear encoder with Linear
        # layer followed by Relu activation function
        # 784 ==> 9
        self.encoder = torch.nn.Sequential(
            torch.nn.Linear(28 * 28, 128),
            torch.nn.ReLU(),
            torch.nn.Linear(128, 64),
            torch.nn.ReLU(),
            torch.nn.Linear(64, 36),
            torch.nn.ReLU(),
            torch.nn.Linear(36, 18),
            torch.nn.ReLU(),
            torch.nn.Linear(18, 9)
        )
         
        # Building an linear decoder with Linear
        # layer followed by Relu activation function
        # The Sigmoid activation function
        # outputs the value between 0 and 1
        # 9 ==> 784
        self.decoder = torch.nn.Sequential(
            torch.nn.Linear(9, 18),
            torch.nn.ReLU(),
            torch.nn.Linear(18, 36),
            torch.nn.ReLU(),
            torch.nn.Linear(36, 64),
            torch.nn.ReLU(),
            torch.nn.Linear(64, 128),
            torch.nn.ReLU(),
            torch.nn.Linear(128, 28 * 28),
            torch.nn.Sigmoid()
        )
 
    def forward(self, x):
        encoded = self.encoder(x)
        decoded = self.decoder(encoded)
        return decoded

model = AE()
model.to(device)


loss_function = torch.nn.MSELoss()

optimizer = torch.optim.Adam(model.parameters(),
                             lr = 1e-1,
                             weight_decay = 1e-8)
epochs = 20
outputs = []
losses = []
st = time.time()
for epoch in range(epochs):
    print("Epoca: ", epoch)
    for (image, _) in loader:
       
        # Reshaping the image to (-1, 784)
        image = image.reshape(-1, 28*28)
        image = image.to(device)       
        # Output of Autoencoder
        reconstructed = model(image)
       
        # Calculating the loss function
        loss = loss_function(reconstructed, image)
       
        # The gradients are set to zero,
        # the gradient is computed and stored.
        # .step() performs parameter update
        optimizer.zero_grad()
        loss.backward()
        optimizer.step()
       
        # Storing the losses in a list for plotting
        #losses.append(loss)
        #outputs.append((epochs, image, reconstructed))
et = time.time()
final=(et-st) / 60
print("tiempo "+str(final))
